@@ -308,6 +308,12 @@ void babel_clean_routing_process(void)
     flush_all_routes();
     babel_interface_close_all();
 
+    /* Clean babel config */
+    diversity_kind = DIVERSITY_NONE;
+    diversity_factor = BABEL_DEFAULT_DIVERSITY_FACTOR;
+    resend_delay = BABEL_DEFAULT_RESEND_DELAY;
+    change_smoothing_half_life(BABEL_DEFAULT_SMOOTHING_HALF_LIFE);
+
     /* cancel events */
     event_cancel(&babel_routing_process->t_read);
     event_cancel(&babel_routing_process->t_update);
