@@ -1,23 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP RD definitions for BGP-based VPNs (IP/EVPN)
  * -- brought over from bgpd/bgp_mplsvpn.c
  * Copyright (C) 2000 Kunihiro Ishiguro <kunihiro@zebra.org>
- *
- * This file is part of FRR.
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with FRR; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
  */
 
 #include <zebra.h>
@@ -29,6 +13,8 @@
 #include "filter.h"
 #include "frrstr.h"
 
+#include "lib/printfrr.h"
+
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_rd.h"
 #include "bgpd/bgp_attr.h"
@@ -97,74 +83,87 @@ void decode_rd_vnc_eth(const uint8_t *pnt, struct rd_vnc_eth *rd_vnc_eth)
 
 int str2prefix_rd(const char *str, struct prefix_rd *prd)
 {
-	int ret;  /* ret of called functions */
-	int lret; /* local ret, of this func */
-	char *p;
-	char *p2;
+	int ret = 0, type = RD_TYPE_UNDEFINED;
+	char *p, *p2;
 	struct stream *s = NULL;
 	char *half = NULL;
 	struct in_addr addr;
-
-	s = stream_new(8);
+	as_t as_val;
 
 	prd->family = AF_UNSPEC;
 	prd->prefixlen = 64;
 
-	lret = 0;
 	p = strchr(str, ':');
 	if (!p)
 		goto out;
 
+	/* a second ':' is accepted */
+	p2 = strchr(p + 1, ':');
+	if (p2) {
+		/* type is in first part */
+		half = XMALLOC(MTYPE_TMP, (p - str) + 1);
+		memcpy(half, str, (p - str));
+		half[p - str] = '\0';
+		type = atoi(half);
+		if (type != RD_TYPE_AS && type != RD_TYPE_IP &&
+		    type != RD_TYPE_AS4)
+			goto out;
+		XFREE(MTYPE_TMP, half);
+		half = XMALLOC(MTYPE_TMP, (p2 - p));
+		memcpy(half, p + 1, (p2 - p - 1));
+		half[p2 - p - 1] = '\0';
+		p = p2 + 1;
+	} else {
+		half = XMALLOC(MTYPE_TMP, (p - str) + 1);
+		memcpy(half, str, (p - str));
+		half[p - str] = '\0';
+	}
 	if (!all_digit(p + 1))
 		goto out;
 
-	half = XMALLOC(MTYPE_TMP, (p - str) + 1);
-	memcpy(half, str, (p - str));
-	half[p - str] = '\0';
+	s = stream_new(RD_BYTES);
 
-	p2 = strchr(str, '.');
-
-	if (!p2) {
-		unsigned long as_val;
-
-		if (!all_digit(half))
-			goto out;
-
-		as_val = atol(half);
-		if (as_val > 0xffff) {
+	/* if it is an AS format or an IP */
+	if (asn_str2asn(half, &as_val)) {
+		if (as_val > UINT16_MAX) {
 			stream_putw(s, RD_TYPE_AS4);
 			stream_putl(s, as_val);
 			stream_putw(s, atol(p + 1));
+			if (type != RD_TYPE_UNDEFINED && type != RD_TYPE_AS4)
+				goto out;
 		} else {
 			stream_putw(s, RD_TYPE_AS);
 			stream_putw(s, as_val);
 			stream_putl(s, atol(p + 1));
+			if (type != RD_TYPE_UNDEFINED && type != RD_TYPE_AS)
+				goto out;
 		}
-	} else {
-		ret = inet_aton(half, &addr);
-		if (!ret)
-			goto out;
-
+	} else if (inet_aton(half, &addr)) {
 		stream_putw(s, RD_TYPE_IP);
 		stream_put_in_addr(s, &addr);
 		stream_putw(s, atol(p + 1));
-	}
+		if (type != RD_TYPE_UNDEFINED && type != RD_TYPE_IP)
+			goto out;
+	} else
+		goto out;
 	memcpy(prd->val, s->data, 8);
-	lret = 1;
+	ret = 1;
 
 out:
 	if (s)
 		stream_free(s);
 	XFREE(MTYPE_TMP, half);
-	return lret;
+	return ret;
 }
 
-char *prefix_rd2str(const struct prefix_rd *prd, char *buf, size_t size)
+char *prefix_rd2str(const struct prefix_rd *prd, char *buf, size_t size,
+		    enum asnotation_mode asnotation)
 {
 	const uint8_t *pnt;
 	uint16_t type;
 	struct rd_as rd_as;
 	struct rd_ip rd_ip;
+	int len = 0;
 
 	assert(size >= RD_ADDRSTRLEN);
 
@@ -174,16 +173,19 @@ char *prefix_rd2str(const struct prefix_rd *prd, char *buf, size_t size)
 
 	if (type == RD_TYPE_AS) {
 		decode_rd_as(pnt + 2, &rd_as);
-		snprintf(buf, size, "%u:%" PRIu32, rd_as.as, rd_as.val);
+		len += snprintfrr(buf + len, size - len, ASN_FORMAT(asnotation),
+				  &rd_as.as);
+		snprintfrr(buf + len, size - len, ":%u", rd_as.val);
 		return buf;
 	} else if (type == RD_TYPE_AS4) {
 		decode_rd_as4(pnt + 2, &rd_as);
-		snprintf(buf, size, "%u:%" PRIu32, rd_as.as, rd_as.val);
+		len += snprintfrr(buf + len, size - len, ASN_FORMAT(asnotation),
+				  &rd_as.as);
+		snprintfrr(buf + len, size - len, ":%u", rd_as.val);
 		return buf;
 	} else if (type == RD_TYPE_IP) {
 		decode_rd_ip(pnt + 2, &rd_ip);
-		snprintf(buf, size, "%s:%" PRIu16, inet_ntoa(rd_ip.ip),
-			 rd_ip.val);
+		snprintfrr(buf, size, "%pI4:%hu", &rd_ip.ip, rd_ip.val);
 		return buf;
 	}
 #ifdef ENABLE_BGP_VNC
@@ -210,6 +212,42 @@ void form_auto_rd(struct in_addr router_id,
 
 	prd->family = AF_UNSPEC;
 	prd->prefixlen = 64;
-	snprintf(buf, sizeof(buf), "%s:%hu", inet_ntoa(router_id), rd_id);
+	snprintfrr(buf, sizeof(buf), "%pI4:%hu", &router_id, rd_id);
 	(void)str2prefix_rd(buf, prd);
 }
+
+static ssize_t printfrr_prd_asnotation(struct fbuf *buf,
+				       struct printfrr_eargs *ea,
+				       const void *ptr,
+				       enum asnotation_mode asnotation)
+{
+	char rd_buf[RD_ADDRSTRLEN];
+
+	if (!ptr)
+		return bputs(buf, "(null)");
+
+	prefix_rd2str(ptr, rd_buf, sizeof(rd_buf), asnotation);
+
+	return bputs(buf, rd_buf);
+}
+
+printfrr_ext_autoreg_p("RDP", printfrr_prd);
+static ssize_t printfrr_prd(struct fbuf *buf, struct printfrr_eargs *ea,
+			    const void *ptr)
+{
+	return printfrr_prd_asnotation(buf, ea, ptr, ASNOTATION_PLAIN);
+}
+
+printfrr_ext_autoreg_p("RDD", printfrr_prd_dot);
+static ssize_t printfrr_prd_dot(struct fbuf *buf, struct printfrr_eargs *ea,
+				const void *ptr)
+{
+	return printfrr_prd_asnotation(buf, ea, ptr, ASNOTATION_DOT);
+}
+
+printfrr_ext_autoreg_p("RDE", printfrr_prd_dotplus);
+static ssize_t printfrr_prd_dotplus(struct fbuf *buf, struct printfrr_eargs *ea,
+				    const void *ptr)
+{
+	return printfrr_prd_asnotation(buf, ea, ptr, ASNOTATION_DOTPLUS);
+}
