@@ -1,21 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PBR-map Header
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *               Donald Sharp
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * Portions:
+ *	Copyright (c) 2023 LabN Consulting, L.L.C.
+ *	Copyright (c) 2021 The MITRE Corporation
  */
 #ifndef __PBR_MAP_H__
 #define __PBR_MAP_H__
@@ -66,6 +56,14 @@ struct pbr_map_interface {
 	bool delete;
 };
 
+enum pbr_forwarding_type {
+	PBR_FT_UNSPEC = 0,
+	PBR_FT_VRF_UNCHANGED,
+	PBR_FT_SETVRF,
+	PBR_FT_NEXTHOP_GROUP,
+	PBR_FT_NEXTHOP_SINGLE,
+};
+
 struct pbr_map_sequence {
 	struct pbr_map *parent;
 
@@ -84,18 +82,65 @@ struct pbr_map_sequence {
 	 */
 	uint32_t ruleno;
 
-	/*
-	 * Our policy Catchers
-	 */
-	struct prefix *src;
-	struct prefix *dst;
-	uint32_t mark;
+
+	/*****************************************************************
+	 * Filter fields
+	 * gpz 230716: I hope to replace all of the filter fields with
+	 * 'struct pbr_filter' from lib/pbr.h.
+	 *****************************************************************/
 
 	/*
-	 * Family of the src/dst.  Needed when deleting since we clear them
+	 * same bit definitions as in lib/pbr.h
 	 */
+	uint32_t filter_bm;
+
+	/* Family of the src/dst. Needed when deleting since we clear them */
 	unsigned char family;
 
+	/* src and dst IP addresses */
+	struct prefix *src;
+	struct prefix *dst;
+
+	/* src and dst UDP/TCP ports */
+	uint16_t src_prt;
+	uint16_t dst_prt;
+
+	uint8_t ip_proto;
+
+	uint8_t match_pcp;
+	uint16_t match_vlan_id; /* bits defined in lib/pbr.h */
+
+	uint16_t match_vlan_flags;
+
+	uint8_t dsfield;
+	uint32_t mark;
+
+	/*****************************************************************
+	 * Action fields
+	 *****************************************************************/
+
+	/*
+	 * same bit definitions as in lib/pbr.h
+	 */
+	uint32_t action_bm;
+
+	union sockunion action_src;
+	union sockunion action_dst;
+
+	uint16_t action_src_port;
+	uint16_t action_dst_port;
+
+	uint8_t action_dscp;
+	uint8_t action_ecn;
+
+	uint8_t action_pcp;
+	uint8_t action_vlan_id;
+
+#define PBR_MAP_UNDEFINED_QUEUE_ID 0
+	uint32_t action_queue_id;
+
+	enum pbr_forwarding_type forwarding_type;
+
 	/*
 	 * Use interface's vrf.
 	 */
@@ -146,18 +191,19 @@ struct pbr_map_sequence {
 #define PBR_MAP_INVALID_BOTH_NHANDGRP    (1 << 3)
 #define PBR_MAP_INVALID_EMPTY            (1 << 4)
 #define PBR_MAP_INVALID_VRF              (1 << 5)
+#define PBR_MAP_INVALID_SET_STRIP_VLAN (1 << 6)
 	uint64_t reason;
 
-	QOBJ_FIELDS
+	QOBJ_FIELDS;
 };
 
-DECLARE_QOBJ_TYPE(pbr_map_sequence)
+DECLARE_QOBJ_TYPE(pbr_map_sequence);
 
 extern struct pbr_map_entry_head pbr_maps;
 
 extern struct pbr_map_sequence *pbrms_get(const char *name, uint32_t seqno);
 extern struct pbr_map_sequence *
-pbrms_lookup_unique(uint32_t unique, ifindex_t ifindex,
+pbrms_lookup_unique(uint32_t unique, char *ifname,
 		    struct pbr_map_interface **ppmi);
 
 extern struct pbr_map *pbrm_find(const char *name);
@@ -168,6 +214,8 @@ extern void pbr_map_add_interface(struct pbr_map *pbrm, struct interface *ifp);
 extern void pbr_map_interface_delete(struct pbr_map *pbrm,
 				     struct interface *ifp);
 
+extern uint8_t pbr_map_decode_dscp_enum(const char *name);
+
 /* Update maps installed on interface */
 extern void pbr_map_policy_interface_update(const struct interface *ifp,
 					    bool state_up);
@@ -194,11 +242,21 @@ extern void pbr_map_check(struct pbr_map_sequence *pbrms, bool changed);
 extern void pbr_map_check_nh_group_change(const char *nh_group);
 extern void pbr_map_reason_string(unsigned int reason, char *buf, int size);
 
-extern void pbr_map_schedule_policy_from_nhg(const char *nh_group);
+extern void pbr_map_schedule_policy_from_nhg(const char *nh_group,
+					     bool installed);
 
 extern void pbr_map_install(struct pbr_map *pbrm);
 
 extern void pbr_map_policy_install(const char *name);
 extern void pbr_map_policy_delete(struct pbr_map *pbrm,
 				  struct pbr_map_interface *pmi);
+
+extern void pbr_map_sequence_delete(struct pbr_map_sequence *pbrms);
+
+extern void pbr_map_check_vrf_nh_group_change(const char *nh_group,
+					      struct pbr_vrf *pbr_vrf,
+					      uint32_t old_vrf_id);
+extern void pbr_map_check_interface_nh_group_change(const char *nh_group,
+						    struct interface *ifp,
+						    ifindex_t oldifindex);
 #endif
