@@ -42,6 +42,7 @@ struct label_manager_chunk {
 	unsigned short instance;
 	uint32_t session_id;
 	uint8_t keep;
+	uint8_t is_dynamic; /* Tell if chunk is dynamic or static */
 	uint32_t start; /* First label of the chunk */
 	uint32_t end;   /* Last label of the chunk */
 };
@@ -61,11 +62,14 @@ DECLARE_HOOK(lm_get_chunk,
 DECLARE_HOOK(lm_release_chunk,
 	     (struct zserv *client, uint32_t start, uint32_t end),
 	     (client, start, end));
+DECLARE_HOOK(lm_write_label_block_config,
+	     (struct vty *vty, struct zebra_vrf *zvrf),
+	     (vty, zvrf));
 DECLARE_HOOK(lm_cbs_inited, (), ());
 
 
-/* declare wrappers to be called in zapi_msg.c (as hooks must be called in
- * source file where they were defined)
+/* declare wrappers to be called in zapi_msg.c or zebra_mpls_vty.c (as hooks
+ * must be called in source file where they were defined)
  */
 void lm_client_connect_call(struct zserv *client, vrf_id_t vrf_id);
 void lm_get_chunk_call(struct label_manager_chunk **lmc, struct zserv *client,
@@ -73,6 +77,7 @@ void lm_get_chunk_call(struct label_manager_chunk **lmc, struct zserv *client,
 		       vrf_id_t vrf_id);
 void lm_release_chunk_call(struct zserv *client, uint32_t start,
 			   uint32_t end);
+int lm_write_label_block_config_call(struct vty *vty, struct zebra_vrf *zvrf);
 
 /* API for an external LM to return responses for requests */
 int lm_client_connect_response(uint8_t proto, uint16_t instance,
@@ -82,7 +87,7 @@ int lm_client_connect_response(uint8_t proto, uint16_t instance,
 /* convenience function to allocate an lmc to be consumed by the above API */
 struct label_manager_chunk *
 create_label_chunk(uint8_t proto, unsigned short instance, uint32_t session_id,
-		   uint8_t keep, uint32_t start, uint32_t end);
+		   uint8_t keep, uint32_t start, uint32_t end, bool is_dynamic);
 void delete_label_chunk(void *val);
 
 /* register/unregister callbacks for hooks */
@@ -95,9 +100,13 @@ void lm_hooks_unregister(void);
  */
 struct label_manager {
 	struct list *lc_list;
+	uint32_t dynamic_block_start;
+	uint32_t dynamic_block_end;
 };
 
 void label_manager_init(void);
+void label_manager_terminate(void);
+
 struct label_manager_chunk *
 assign_label_chunk(uint8_t proto, unsigned short instance, uint32_t session_id,
 		   uint8_t keep, uint32_t size, uint32_t base);
