@@ -2252,6 +2252,9 @@ static void lib_vrf_mpls_fec_nexthop_resolution_cli_write(
 	}
 }
 
+#if CONFDATE > 20251207
+CPP_NOTICE("Remove no-op netns command")
+#endif
 DEFPY_YANG (vrf_netns,
        vrf_netns_cmd,
        "[no] netns ![NAME$netns_name]",
